//-----------------------------------------------------------------------------
// (C) Copyright SDG Electronics 2019
//
// Project Reference:
//     Vacuum Pick-Up Tool
//-----------------------------------------------------------------------------
// DESCRIPTION
//     Vacuum Pick-Up Tool Controller - PIC18F13K22 MCU
//        Main Program
//
// ORIGINAL AUTHOR
//     Steven Gardner
//     23rd July 2019
//-----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <xc.h>
//-----------------------------------------------------------------------------
// Configuration Words
//-----------------------------------------------------------------------------
// CONFIG1H
#pragma config FOSC = IRC       // Oscillator Selection bits (Internal RC oscillator)
#pragma config PLLEN = OFF      // 4 X PLL Enable bit (PLL is under software control)
#pragma config PCLKEN = ON      // Primary Clock Enable bit (Primary clock enabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF       // Internal/External Oscillator Switchover bit (Oscillator Switchover mode disabled)

// CONFIG2L
#pragma config PWRTEN = ON      // Power-up Timer Enable bit (PWRT enabled)
#pragma config BOREN = SBORDIS  // Brown-out Reset Enable bits (Brown-out Reset enabled in hardware only (SBOREN is disabled))
#pragma config BORV = 22        // Brown Out Reset Voltage bits (VBOR set to 2.2 V nominal)

// CONFIG2H
#pragma config WDTEN = ON       // Watchdog Timer Enable bit (WDT is always enabled. SWDTEN bit has no effect.)
#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits (1:32768)

// CONFIG3H
#pragma config HFOFST = ON      // HFINTOSC Fast Start-up bit (HFINTOSC starts clocking the CPU without waiting for the oscillator to stablize.)
#pragma config MCLRE = ON       // MCLR Pin Enable bit (MCLR pin enabled, RA3 input pin disabled)

// CONFIG4L
#pragma config STVREN = ON      // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
#pragma config LVP = OFF        // Single-Supply ICSP Enable bit (Single-Supply ICSP disabled)
#pragma config BBSIZ = OFF      // Boot Block Size Select bit (512W boot block size)
#pragma config XINST = OFF      // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled (Legacy mode))

// CONFIG5L
#pragma config CP0 = OFF        // Code Protection bit (Block 0 not code-protected)
#pragma config CP1 = OFF        // Code Protection bit (Block 1 not code-protected)

// CONFIG5H
#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot block not code-protected)
#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF       // Write Protection bit (Block 0 not write-protected)
#pragma config WRT1 = OFF       // Write Protection bit (Block 1 not write-protected)

// CONFIG6H
#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers not write-protected)
#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot block not write-protected)
#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF      // Table Read Protection bit (Block 0 not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF      // Table Read Protection bit (Block 1 not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot block not protected from table reads executed in other blocks)
//-----------------------------------------------------------------------------
// Function Prototypes
//-----------------------------------------------------------------------------
void app_init(void);
void delay_100ms(void);
//-----------------------------------------------------------------------------
#define TRUE            1
#define FALSE           0
#define INPUT           1
#define OUTPUT          0

#define ADC_ARRAY_SIZE	8

#define D_PEDAL_IN       TRISBbits.TRISB6
#define D_POTENTIOMETER  TRISBbits.TRISB4
#define D_LED_RED        TRISCbits.TRISC3
#define D_LED_GRN        TRISCbits.TRISC5
#define D_LED_BLU        TRISCbits.TRISC4
#define D_RELAY_PUMP     TRISCbits.TRISC0
#define D_RELAY_SOLENOID TRISCbits.TRISC1

#define PEDAL_IN       	PORTBbits.RB6
#define POTENTIOMETER  	PORTBbits.RB4
#define LED_RED        	LATCbits.LATC5
#define LED_GRN        	LATCbits.LATC3
#define LED_BLU        	LATCbits.LATC4
#define RELAY_PUMP     	LATCbits.LATC0
#define RELAY_SOLENOID 	LATCbits.LATC1

//-----------------------------------------------------------------------------
// Global Data
//-----------------------------------------------------------------------------
uint8_t g_adcArray[9] = {0};
uint8_t g_adcArrayIndex = 0;
uint8_t g_adcArrayAvg = 0;
uint16_t g_adcArraySum = 0;
uint8_t g_pedalPreviousState = 0;

uint16_t g_pumpCountdown = 0;           // Realtime Counter
uint16_t g_pumpCountdownTarget = 0;     // Target Value for timeout
uint16_t g_pumpCountdownReached = 0;    // Set to 1 when timeout reached
//-----------------------------------------------------------------------------
// Main program loop
//-----------------------------------------------------------------------------
int main(int argc, char** argv) {

    app_init();
    while(1)
    {
        CLRWDT();
        if(PEDAL_IN)            // Footpedal is normally high, low if pedal pressed
        {
            RELAY_SOLENOID = 0;
                                // Check if this was a pedal transition
            if(g_pedalPreviousState == 1)
            {
                                // Yes, was previously depressed, so change state
                g_pedalPreviousState = 0;
                                // Now do everything on the transition from pedal pressed to not pressed
                
                                // Reset timeouts...
                g_pumpCountdownReached = 0; 
                g_pumpCountdown = 0;
                
            }
                                // If the pump countdown has timed out (timed in the interrupt) then turn off pump
            if(g_pumpCountdownReached == 1)
            {
                RELAY_PUMP = 0; 
                
                LED_RED             =   1;
                LED_GRN             =   0;
                LED_BLU             =   0;
            }
        }
        else
        {
            RELAY_PUMP = 1;     // Whenever pedal is pressed, make sure solenoid and pump are depressed
            RELAY_SOLENOID = 1;
            
            LED_RED             =   0;
            LED_GRN             =   1;
            LED_BLU             =   0;
            
            g_pedalPreviousState = 1;
            g_pumpCountdownReached = 0; 
            g_pumpCountdown = 0;
            
        }
    
    }
    return (EXIT_SUCCESS);
}

//-------------------------------------------------------------------------
//  NAME
//      app_init
//
//  DESCRIPTION
//!     Initialises module
//
//  RETURNS
//! @return void
//
//-------------------------------------------------------------------------
void app_init(void)
{
    CLRWDT();
    // Configure Oscillator Block
    OSCCON  =   0x30;           // Internal Oscillator 1MHz
    OSCTUNE =   0x00;           // No PLL
    // Configure Analogue Pins
    ANSEL   =   0x00;           // Enable all digital input buffers
    ANSELH  =   0x04;           // Analogue input on AN10

    // Set GPIO
	
	D_PEDAL_IN       =   INPUT;
	D_POTENTIOMETER  =   INPUT;
	D_LED_RED        =   OUTPUT;
	D_LED_GRN        =   OUTPUT;
	D_LED_BLU        =   OUTPUT;
	D_RELAY_PUMP     =   OUTPUT;
	D_RELAY_SOLENOID =   OUTPUT;
	

    LED_RED         =   1;
    LED_GRN         =   0;
    LED_BLU         =   0;
    RELAY_PUMP      =   0;
    RELAY_SOLENOID  =   0;
	
	// Configure ADC
	ADCON2			=	0b00100011;	// Left Justified, 8TAD, FRC conversion clock
	ADCON1			=	0;			// Vref Vdd, Vss
	ADCON0bits.CHS	=   0b1010;		// Use AN10
	ADCON0bits.ADON =   1;			// ADC On
	
	
    // Timer 1 RTC Configuration

    // Timer 2 Configuration
	PR2		=	156;
	T2CON 	= 	0b0000110;

    // Global Variable Initialisations

    CLRWDT();
    
    // Enable Interrupts
    PIE1bits.TMR1IE =   0b0;    // Enable TMR1 Interrupt
    PIE1bits.TMR2IE =   0b1;    // Enable TMR2 Interrupt
	PIE1bits.ADIE	=	0b1;	// Enable ADC Interrupt
    INTCONbits.GIE  =   0b1;    // Enable Interrupts
    INTCONbits.PEIE =   0b1;
    
    ADCON0bits.GODONE 	=	1;  // Start the ADC
    
    // Do soft start
    // 1MHz Fosc, 250kHz Fcyc, 4us Tcyc
    // 100ms = 25000 cycles
    // 250ms = 62500 cycles
    // 65535-25000 = 40535  = 0x9E57
    // 65535-62500 = 3035   = 0x0BDB
    T1CON               =   0;
    PIR1bits.TMR1IF     =   0b0;
    T1CONbits.T1CKPS    =   0b01;
    TMR1                =   0x0BDB; // 500ms delay
    T1CONbits.TMR1ON    =   0b1;
    while(!(PIR1bits.TMR1IF));
    T1CON               =   0;
    
    CLRWDT();
                                    // Turn on SS relay


}

//-------------------------------------------------------------------------
//  NAME
//      delay_100ms
//
//  DESCRIPTION
//!     Delay 100ms
//
//  RETURNS
//! @return void
//
//-------------------------------------------------------------------------
void delay_100ms(void)
{
    CLRWDT();
    
    T1CON               =   0;
    PIR1bits.TMR1IF     =   0b0;
    T1CONbits.T1CKPS    =   0b00;
    TMR1                =   0x9E57; // 100ms delay
    T1CONbits.TMR1ON    =   0b1;
    while(!(PIR1bits.TMR1IF));
    T1CON               =   0;
    
    CLRWDT();
}

//-----------------------------------------------------------------------------
// Interrupt
//-----------------------------------------------------------------------------
void __interrupt () my_isr_routine (void)                    // Interrupt Handler
{
    //-------------------------------------------------------------------------
    // Timer 1 RTC Interrupt
    //-------------------------------------------------------------------------
    if(PIR1bits.TMR1IF && PIE1bits.TMR1IE)  // If Timer 1 flag is set & Interrupt is enabled
    {                                       // RTC Interrupt
        asm("bsf TMR1H,6");                 // Set upper two MSBs of timer for 0.5s rollover
        asm("bsf TMR1H,7");                 
        PIR1bits.TMR1IF       =   0;        // Clear Timer 1 Interrupt Flag bit

    }
    //-------------------------------------------------------------------------
    // Timer 2 ~10ms Interrupt
    //-------------------------------------------------------------------------
    if(PIR1bits.TMR2IF && PIE1bits.TMR2IE)  // If Timer 2 flag is set & Interrupt is enabled
    {   
        LED_BLU = 1;
        PIR1bits.TMR2IF     =   0;          // Clear TMR2 Interrupt Flag
        ADCON0bits.GODONE 	=	1;
        
        if(g_pumpCountdown > g_pumpCountdownTarget)             // Set flag if timeout has been reached.
        {
            g_pumpCountdownReached = 1;
        }
        else
        {
            g_pumpCountdown++;
        }
        LED_BLU = 0;

    }
	//-------------------------------------------------------------------------
    // ADC Interrupt
    //-------------------------------------------------------------------------
    if(PIR1bits.ADIF && PIE1bits.ADIE)  // If Timer 2 flag is set & Interrupt is enabled
    {                                      
        
		g_adcArraySum = g_adcArraySum - g_adcArray[g_adcArrayIndex];
        g_adcArray[g_adcArrayIndex] = ADRESH;
        g_adcArraySum = g_adcArraySum + g_adcArray[g_adcArrayIndex];
        g_adcArrayAvg = (uint16_t)(g_adcArraySum/ADC_ARRAY_SIZE);
        g_adcArrayIndex++;
        if(g_adcArrayIndex >= ADC_ARRAY_SIZE) 
        {
            g_adcArrayIndex = 0;
        }
        // Convert average to a countdown value
        // 0-255 into 0-32768
        g_pumpCountdownTarget = g_adcArrayAvg * 128;

		PIR1bits.ADIF     	=   0;      // Clear ADC Interrupt Flag
		
		
    }
}



